import re
import os
import time
import socket

def get_tm(text):
    year=text[0:4]
    month=text[4:6]
    day=text[6:8]
    hour=text[8:10]
    minute=text[10:12]
    second=text[12:14]

    tm=[]
    tm.append(int(year))
    tm.append(int(month))
    tm.append(int(day))
    tm.append(int(hour))
    tm.append(int(minute))
    tm.append(int(second))
    tm.append(0)
    tm.append(0)
    tm.append(-1)

    return tm

def get_seconds(tm):

    # fix time zone to avoid DST jumps
    if os.getenv("TZ")==None:
        os.environ["TZ"]="UTC"

    return int(time.mktime(tm))

def get_item(text):
    return re.split('=',text)[1]

def parse_entry(entry):

    #
    # Be robust against offset pointing into the middle of a record,
    # which can happen when the service is stopped and not restarted
    # until the gridftp logfile has been rotated; in that case we
    # could process all entries from the start of the current file,
    # but if ever there is an entry corrupt due to other causes, we
    # would keep hitting it, so we just sync with the next entry and
    # continue from there.  If the service was explicitly stopped for
    # a long time and then restarted, it is quite reasonable that only
    # new transfers will be considered for monitoring.
    #
 
    if entry[0:5] != 'DATE=':
        return "ERROR"

    list = re.split('\s+',entry)

    #
    # The CASTOR gridftp server has been observed not always to log a
    # "USER" field; that is a bug, but we shall be robust against it.
    #

    if list[5][0:5] != 'USER=':
        list[5:5] = ['USER=unknown']
            
    username = get_item(list[5])
    ftp_file = get_item(list[6])

    # We don't record things like edg-gridftp-ls and such
    if ftp_file == '/bin/ls':
        return "Ignore"

    start=get_item(list[4])
    startTimeList=re.split('\.',start)
    startDateTime=startTimeList[0]
    startMicroseconds=startTimeList[1]
    start_tm=get_tm(startDateTime)
    start_seconds=get_seconds(start_tm)
    start_microseconds=int(startMicroseconds)

    end=get_item(list[0])
    endTimeList=re.split('\.',end)
    endDateTime=endTimeList[0]
    endMicroseconds=endTimeList[1]
    end_tm=get_tm(endDateTime)
    end_seconds=get_seconds(end_tm)
    end_microseconds=int(endMicroseconds)

    difference_microseconds=0.000001*(end_microseconds - start_microseconds)
    difference_seconds=end_seconds - start_seconds
    difftime=difference_seconds + difference_microseconds

    if difftime <= 0:
        # correct for DST ending
        difftime = difftime + 3600

    # ensure fully qualified host name
    t=socket.gethostbyname(get_item(list[1]))
    host=socket.gethostbyaddr(t)[0]
    peer=get_item(list[13])[2:-1]

    try:
        tmp=socket.gethostbyaddr(peer)[0]
        peer=tmp
    except socket.herror,msg:
        pass

    if get_item(list[14]) == 'STOR':
        src=peer
        dest=host
    else:
        src=host
        dest=peer
    
    block=get_item(list[8])
    streams=get_item(list[11])
    tb=get_item(list[9])
    bytes=int(tb)
    stripes=get_item(list[12])
    mega=1048576
    tp="%.4f" % (float(tb)/(difftime*mega))
    transfer_rate=tp
    buffer=get_item(list[7])
    code=get_item(list[15])
    volume=get_item(list[10])
        
    end_date =   "%u-%02u-%02u" % (end_tm[0], end_tm[1], end_tm[2])
    end_time = "%02u:%02u:%02u" % (end_tm[3], end_tm[4], end_tm[5])
        
    thislist=[]
    thislist.append("'"+host+"'")
    thislist.append("'"+username+"'")
    thislist.append("'"+src+"'")
    thislist.append("'"+dest+"'")
    thislist.append(block)
    thislist.append(streams)
    thislist.append(buffer)
    thislist.append(bytes)
    thislist.append(stripes)
    thislist.append(transfer_rate)
    thislist.append("'"+volume+"'")
    thislist.append("'"+ftp_file+"'")
    thislist.append(code)
    thislist.append(start_seconds)
    thislist.append(start_microseconds)
    thislist.append(end_seconds)
    thislist.append(end_microseconds)
    thislist.append("'" + end_date + "'")
    thislist.append("'" + end_time + "'")
    
    command = "INSERT INTO  GridftpMonitor (" + \
              "host," + \
              "user_name," + \
              "src," + \
              "dest," + \
              "block_size," + \
              "nstreams," + \
              "buffer_size," + \
              "nbytes," + \
              "nstripes," + \
              "transfer_rate," + \
              "volume," + \
              "file_name," + \
              "code," + \
              "start_time," + \
              "start_time_micros," + \
              "end_time," + \
              "end_time_micros," + \
              "MeasurementDate," + \
              "MeasurementTime) VALUES ("

    for i in thislist:
        command = command + str(i) + ","
    command = command[0:-1] + ");"

    return command
